(ns wanijo.schema.domain
  (:require [clojure.spec.alpha :as spec]
            [wanijo.framework.neo4j :as neo4j]))

(spec/def ::name
  (spec/and string? not-empty))

(spec/def ::created-at
  (spec/or :int (spec/and int?
                          #(> % 20000101000000))
           :str (spec/and string?
                          #(re-matches #"\d{4}\d{2}\d{2}\d{2}\d{2}\d{2}" %))))

(neo4j/defquery
  all-created-by
  "MATCH (s:schema)-[:created_by]->(u:user)
   WHERE u.uuid = {uuid}
   RETURN s
   ORDER BY s.name")

(defn all-created-by! [user-uuid]
  (map :s
       (neo4j/exec-query
        all-created-by
        {:uuid user-uuid})))

(neo4j/defquery
  create-new
  "MATCH (u:user)
   WHERE u.uuid = {u_uuid}
   CREATE (s:schema)-[:created_by]->(u)
   SET s.name = {name}
   SET s.uuid = {s_uuid}
   RETURN s")

(defn create-new! [name user-uuid]
  (->>
   (neo4j/exec-query
    create-new
    {:u_uuid user-uuid
     :name name
     :s_uuid (neo4j/uuid)})
   first
   :uuid))

(neo4j/defquery
  find-by-id!
  "MATCH (s:schema)
   WHERE s.uuid = {uuid}
   RETURN s")

(defn find-by-uuid! [uuid]
  (->>
   (neo4j/exec-query
    find-by-id!
    {:uuid uuid})
   first
   :s))
